<?php
/**
 * Main Plugin
 *
 * @package   dorzki\WooCommerce\CustomProductField
 * @author    Dor Zuberi <admin@dorzki.io>
 * @copyright 2023 dorzki
 * @version   1.0.0
 */

namespace dorzki\WooCommerce\CustomProductField;

use Throwable;

// Block if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Class Plugin
 *
 * @package dorzki\WooCommerce\CustomProductField
 */
class Plugin {
	const FIELD_KEY = 'dorzki_wc_custom_field';


	/**
	 * Plugin constructor
	 */
	public function __construct() {
		add_action( 'woocommerce_before_add_to_cart_button', [ $this, 'inject_field' ] );
		add_action( 'woocommerce_add_order_item_meta', [ $this, 'save_order_item_data' ], 10, 2 );

		add_filter( 'woocommerce_add_to_cart_validation', [ $this, 'validate_field' ], 10, 2 );
		add_filter( 'woocommerce_add_cart_item_data', [ $this, 'save_field' ] );
		add_filter( 'woocommerce_get_cart_item_from_session', [ $this, 'store_data' ], 10, 2 );
		add_filter( 'woocommerce_get_item_data', [ $this, 'show_on_cart' ], 10, 2 );
		add_filter( 'woocommerce_order_item_display_meta_key', [ $this, 'show_on_overview' ] );
		add_filter( 'woocommerce_email_order_meta_fields', [ $this, 'show_on_email' ] );
	}


	/**
	 * Inject the field just above the add to cart button
	 */
	public function inject_field() {
		$field_value = ( isset( $_POST[ self::FIELD_KEY ] ) ) ? sanitize_text_field( wp_unslash( $_POST[ self::FIELD_KEY ] ) ) : null;

		printf(
			'<label for="%1$s">%2$s</label><input type="text" id="%1$s" name="%1$s" value="%3$s">',
			esc_attr( self::FIELD_KEY ),
			esc_html__( 'Your Name:', 'dorzki-wc-product-custom-field' ),
			esc_attr( $field_value )
		);
	}

	/**
	 * Saves the order data to database
	 *
	 * @param int   $item_id order item id
	 * @param array $values order item data
	 */
	public function save_order_item_data( int $item_id, array $values ) {
		if ( ! empty( $values[ self::FIELD_KEY ] ) ) {
			try {
				wc_add_order_item_meta( $item_id, self::FIELD_KEY, $values[ self::FIELD_KEY ] );
			} catch ( Throwable $ex ) {
				return;
			}
		}
	}

	/**
	 * Check if user have entered data into the field
	 *
	 * @param bool $passed if product passed validation
	 * @param int  $product_id product id
	 *
	 * @return bool
	 */
	public function validate_field( bool $passed, int $product_id ): bool {
		$field_value = ( isset( $_POST[ self::FIELD_KEY ] ) ) ? sanitize_text_field( wp_unslash( $_POST[ self::FIELD_KEY ] ) ) : null;

		if ( empty( $field_value ) ) {
			$product = wc_get_product( $product_id );

			wc_add_notice(
				sprintf(
				/* translators: 1: Product Name */
					esc_html__( '"%1$s" can\'t be added to cart until you fill all data.', 'dorzki-wc-product-custom-field' ),
					"<strong>{$product->get_title()}</strong>"
				)
			);

			return false;
		}

		return $passed;
	}

	/**
	 * Save the field value to current cart session
	 *
	 * @param array $cart_item cart item data
	 *
	 * @return array
	 */
	public function save_field( array $cart_item ): array {
		$field_value = ( isset( $_POST[ self::FIELD_KEY ] ) ) ? sanitize_text_field( wp_unslash( $_POST[ self::FIELD_KEY ] ) ) : null;

		if ( ! empty( $field_value ) ) {
			$cart_item[ self::FIELD_KEY ] = $field_value;
		}

		return $cart_item;
	}

	/**
	 * Stores cart item field data to current cart session
	 *
	 * @param array $cart_item_data cart item data
	 * @param array $values product item data
	 *
	 * @return array
	 */
	public function store_data( array $cart_item_data, array $values ): array {
		if ( isset( $values[ self::FIELD_KEY ] ) ) {
			$cart_item_data[ self::FIELD_KEY ] = $values[ self::FIELD_KEY ];
		}

		return $cart_item_data;
	}

	/**
	 * Display custom field data on cart page
	 *
	 * @param array $item_data item registered data
	 * @param array $cart_item item saved data
	 *
	 * @return array
	 */
	public function show_on_cart( array $item_data, array $cart_item ): array {
		if ( ! empty( $cart_item[ self::FIELD_KEY ] ) ) {
			$item_data[] = [
				'name'  => esc_html__( 'Your Name', 'dorzki-wc-product-custom-field' ),
				'value' => sanitize_text_field( $cart_item[ self::FIELD_KEY ] ),
			];
		}

		return $item_data;
	}

	/**
	 * Change the admin meta display key
	 *
	 * @param string $display_key meta display key
	 *
	 * @return string
	 */
	public function show_on_overview( string $display_key ): string {
		return ( self::FIELD_KEY === $display_key ) ? __( 'Your Name', 'dorzki-wc-product-custom-field' ) : $display_key;
	}

	/**
	 * Display custom field data on email
	 *
	 * @param array $fields array of fields
	 *
	 * @return array
	 */
	public function show_on_email( array $fields ): array {
		$fields[ self::FIELD_KEY ] = __( 'Your Name', 'dorzki-wc-product-custom-field' );

		return $fields;
	}
}

// Init plugin
new Plugin();
